﻿var request = require("request");

exports.handler = function (event, context, callback) {

    // Purpose:
    // This script is designed to send new customers from new orders to a MailChimp list

    // Configuration
    // This script should receive an order:created event with expand=customer

    // Environment Variables to be configured in the Hosted Function:

    // mailchimp_api_key: The MailChimp API key. Get an API key in MailChimp by clicking on
    // your profile in the upper right of the MailChimp control panel, select Account>
    // Extras> API keys

    // mailchimp_list_id: The MailChimp list or lists you want the customer to be
    // subscribed to. The list ID can be found in MailChimp. Navigate to the list, go to
    // Settings> List Name and Defaults, look for the property "List ID".

    // Define the items in the request that we might want to use.
    var headers = event.headers;
    var queryStringParameters = event.queryStringParameters;
    var body = JSON.parse(event.body);
    var ev = process.env;

    // We are going to configure this hosted function to be invoked by a Comecero event
    // notification. The order is contained in the event notification's "data" property.
    var order = body.data;

    // This is a function we'll use to send the response back to the handler when
    // we've finished processing the request.
    var sendResponse = function (statusCode, outputBody) {

        // The response should include the following:
        // statusCode: The HTTP status code (i.e. 200)
        // headers: Headers to include in the HTTP response to the caller
        // body: The response body (optional - can be null or a string)
        var response = {
            statusCode: statusCode,
            body: null,
            headers: {
                "Content-Type": "application/json"
            }
        };

        if (outputBody) {
            response.body = JSON.stringify(outputBody)
        }

        // The format of the callback function is: callback(error, success)
        // Generally even if an error scenario you should pass through a success response
        // with the error condition in the response (such as statusCode: 400 with the
        // error as the body). If you send an error to the callback, the response HTTP status
        // code to the caller will always be 502.
        callback(null, response);

    };

    // Set up the MailChimp request payload
    var payload = {
        email_address: order.customer.email,
        status: "subscribed",
        merge_fields: {}
    }

    // If you have the customer's name, split into a first and last name.
    if (order.customer.name) {
        var nameSplit = order.customer.name.split(" ");
        // Add to the payload to include with the subscriber's profile.
        payload.merge_fields.FNAME = nameSplit[0].trim();
        if (nameSplit.length > 1)
            payload.merge_fields.LNAME = nameSplit[1].trim();
    }

    // The host for the request is the characters after the last dash in the api key.
    var host = ev.mailchimp_api_key.substring(ev.mailchimp_api_key.lastIndexOf("-") + 1);

    // Set up your api request options
    var options = {
        url: "https://" + host + ".api.mailchimp.com/3.0/lists/" + ev.mailchimp_list_id + "/members",
        auth: {
            // The MailChimp API does not require a username, but you must pass through some (any) string
            user: "not-used",
            pass: ev.mailchimp_api_key,
        },
        body: payload,
        json: true
    };

    // Send the request
    request.post(options, function (error, response) {
        if (!error && response.statusCode >= 200 && response.statusCode < 300) {
            // Pass through the response from MailChimp to the caller.
            sendResponse(200, response.body);
            return;
        } else {
            if (response && response.body) {
                // Pass through the error response from MailChimp to the caller.
                sendResponse(response.statusCode, response.body);
                return;
            } else {
                // Pass through whatever error was supplied to the caller.
                sendResponse(502, error);
                return;
            }
        }
    });

};